/* Copyright (C) 2009 Registro.br. All rights reserved. 
* 
* Redistribution and use in source and binary forms, with or without 
* modification, are permitted provided that the following conditions are 
* met:
* 1. Redistribution of source code must retain the above copyright 
*    notice, this list of conditions and the following disclaimer.
* 2. Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in the
*    documentation and/or other materials provided with the distribution.
* 
* THIS SOFTWARE IS PROVIDED BY REGISTRO.BR ``AS IS'' AND ANY EXPRESS OR
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIE OF FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
* EVENT SHALL REGISTRO.BR BE LIABLE FOR ANY DIRECT, INDIRECT,
* INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
* OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
* TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
* USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
* DAMAGE.
 */
package br.registro.dnsshim.xfrd.dao.filesystem;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;

import org.apache.log4j.Logger;

import br.registro.dnsshim.common.repository.FileSystemUtil;
import br.registro.dnsshim.xfrd.domain.SlaveConfig;
import br.registro.dnsshim.xfrd.domain.SlaveConfig.ConfigOperation;
import br.registro.dnsshim.xfrd.repository.SlaveConfigRepository;

public class SlaveConfigDao implements SlaveConfigRepository {
	private static final Logger logger = Logger.getLogger(SlaveConfigDao.class);

	public static final String EXTENSION = ".config";
	public static final String EXTENSION_REMOVED = ".removed";
	
	private final String BASE_DIR;

	public SlaveConfigDao() {
		StringBuilder root = new StringBuilder();
		try {
 			// {DNSSHIM_HOME|user.home}/dnsshim/conf/xfrd/slaves/<groupname>.config
			if (System.getenv("DNSSHIM_HOME") == null) {
				root.append(System.getProperty("user.home"));
				root.append(File.separatorChar);
				root.append("dnsshim");
			} else {
				root.append(System.getenv("DNSSHIM_HOME"));
			}
			root.append(File.separatorChar);
			root.append("xfrd");
			root.append(File.separatorChar);
			root.append("conf");
			root.append(File.separatorChar);
			root.append("slaves");
			root.append(File.separatorChar);

			File directory = new File(root.toString());
			FileSystemUtil.makePath(directory);
			
		} catch (IOException e) {
			logger.error(e.getMessage(), e);
		}
		BASE_DIR = root.toString(); 
	}

	@Override
	public void save(SlaveConfig slaveConfig) throws IOException {
		if (logger.isDebugEnabled()) {
			logger.debug("Saving SlaveConfig...");
		}
		
		StringBuilder path = new StringBuilder(BASE_DIR);
		path.append(slaveConfig.getGroupName());
		if (slaveConfig.getConfigOperation() == ConfigOperation.CURRENT) {
			path.append(EXTENSION);
		} else if (slaveConfig.getConfigOperation() == ConfigOperation.REMOVED) {
			path.append(EXTENSION_REMOVED);
		}
		
		File file = new File(path.toString());
		BufferedWriter bufferedWriter = new BufferedWriter(new FileWriter(file));
		bufferedWriter.write(slaveConfig.getOutput());
		bufferedWriter.close();
	}
	
	@Override
	public String getPath() {
		return BASE_DIR;
	}
}
