/* Copyright (C) 2009 Registro.br. All rights reserved. 
* 
* Redistribution and use in source and binary forms, with or without 
* modification, are permitted provided that the following conditions are 
* met:
* 1. Redistribution of source code must retain the above copyright 
*    notice, this list of conditions and the following disclaimer.
* 2. Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in the
*    documentation and/or other materials provided with the distribution.
* 
* THIS SOFTWARE IS PROVIDED BY REGISTRO.BR ``AS IS'' AND ANY EXPRESS OR
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIE OF FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
* EVENT SHALL REGISTRO.BR BE LIABLE FOR ANY DIRECT, INDIRECT,
* INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
* OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
* TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
* USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
* DAMAGE.
 */
package br.registro.dnsshim.xfrd.dao.filesystem;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

import org.apache.log4j.Logger;

import br.registro.dnsshim.common.repository.FileSystemUtil;
import br.registro.dnsshim.xfrd.domain.User;
import br.registro.dnsshim.xfrd.repository.UserRepository;

public class UserDao implements UserRepository {
	private static final Logger logger = Logger.getLogger(UserDao.class);
	private static final String EXTENSION = ".properties";
	private static File file;
	
	public UserDao() {
		StringBuilder root = new StringBuilder();
		try {
			// {DNSSHIM_HOME|user.home}/dnsshim/conf/xfrd/users.properties
			if (System.getenv("DNSSHIM_HOME") == null) {
				root.append(System.getProperty("user.home"));
				root.append(File.separatorChar);
				root.append("dnsshim");
			} else {
				root.append(System.getenv("DNSSHIM_HOME"));
			}
			root.append(File.separatorChar);
			root.append("xfrd");
			root.append(File.separatorChar);
			root.append("conf");
			root.append(File.separatorChar);

			File directory = new File(root.toString());
			FileSystemUtil.makePath(directory);

			root.append("users");
			root.append(EXTENSION);

			file = new File(root.toString());
			if (!file.exists()) {
				if (!file.createNewFile()) {
					throw new IOException("Cannot create file:"+file);
				}
			}

			if (logger.isDebugEnabled()) {
				logger.debug("XFRD users file:" + file);
			}

		} catch (IOException e) {
			logger.error(e.getMessage(), e);
		}
	}
	
	@Override
	public void save(User user) throws IOException {
		if (logger.isDebugEnabled()) {
			logger.debug("Saving user...");
		}
		
		Properties conf = new Properties();
		FileInputStream fis = new FileInputStream(file);
		conf.load(fis);
		conf.put(user.getUsername(), user.getPassword());
		FileWriter fw = new FileWriter(file);
		conf.store(fw, "Generated by DNSSHIM");
		fis.close();
		fw.close();
	}

	@Override
	public User findByUsername(String username) throws IOException {
		if (logger.isDebugEnabled()) {
			logger.debug("Loading user");
		}
		
		Properties conf = new Properties();
		FileInputStream fis = new FileInputStream(file);
		conf.load(fis);
		String password = conf.getProperty(username);
		
		fis.close();
		// found
		if (password != null) {
			User user = new User();
			user.setUsername(username);
			user.setPassword(password);
			return user;
		}
		return null;
	}
	
	@Override
	public Map<String, User> loadAll() throws IOException {
		if (logger.isInfoEnabled()) {
			logger.info("Loading users");
		}
		
		Properties conf = new Properties();
		FileInputStream fis = new FileInputStream(file);
		conf.load(fis);
		
		Map<String, User> users = new HashMap<String, User>();
		for (Object key : conf.keySet()) {
			String k = (String) key;
			String password = conf.getProperty(k);
			User user = new User();
			user.setUsername(k);
			user.setPassword(password);
			users.put(k, user);
		}
		fis.close();
		return users;
	}
}