/* Copyright (C) 2009 Registro.br. All rights reserved. 
* 
* Redistribution and use in source and binary forms, with or without 
* modification, are permitted provided that the following conditions are 
* met:
* 1. Redistribution of source code must retain the above copyright 
*    notice, this list of conditions and the following disclaimer.
* 2. Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in the
*    documentation and/or other materials provided with the distribution.
* 
* THIS SOFTWARE IS PROVIDED BY REGISTRO.BR ``AS IS'' AND ANY EXPRESS OR
* IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIE OF FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
* EVENT SHALL REGISTRO.BR BE LIABLE FOR ANY DIRECT, INDIRECT,
* INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
* BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
* OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
* ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
* TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
* USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
* DAMAGE.
 */
package br.registro.dnsshim.xfrd.domain;

import java.util.HashMap;
import java.util.LinkedHashSet;
import java.util.Map;
import java.util.Set;

import br.registro.dnsshim.domain.logic.Cacheable;

public class SlaveGroup implements Cacheable<String> {
	private String name;
	private Map<String, Slave> slaves = new HashMap<String, Slave>();
	private Set<Zonename> zonenames = new LinkedHashSet<Zonename>(); 
	private Set<Zonename> removedZonenames = new LinkedHashSet<Zonename>();

	public Slave addSlave(Slave slave) {
		return slaves.put(slave.getAddress().getHostAddress(), slave);
	}

	public Slave removeSlave(String address) {
		return slaves.remove(address);
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}
	
	public boolean hasSlave(String address) {
		return slaves.containsKey(address);
	}

	public Map<String, Slave> getSlaves() {
		return new HashMap<String, Slave>(slaves);
	}
	
	public boolean addZonename(Zonename zonename) {
		removedZonenames.remove(zonename);
		return zonenames.add(zonename);
	}
	
	public boolean removeZonename(Zonename zonename) {
		zonenames.remove(zonename);
		return removedZonenames.add(zonename);
	}
	
	public synchronized Set<Zonename> getZonenames() {
		return new LinkedHashSet<Zonename>(zonenames);
	}
	
	public synchronized Set<Zonename> getRemovedZonenames() {
		return new LinkedHashSet<Zonename>(removedZonenames);
	}
	
	public void clearRemovedZonenames() {
		removedZonenames.clear();
	}
	
	@Override
	public String getCacheKey() {
		return name;
	}
}